#!/usr/bin/python
import os
from subprocess import call
import time


# Look at ssh config file and find which are the properly configured accesses to Marenostrum 3
def all_passwordless_access():
    # Place where the config file usually is
    filename = "~/.ssh/config"
    filename = os.path.expanduser(filename)
    # Assert config file exists in usual place.
    assert os.path.isfile(filename), "Config file not found in %s" % filename

    # Create empty lists for connection names, hosts and user-names.
    _name = []
    _host = []
    _user = []

    # Open the file to read the content and save the info into the arrays.
    with open(filename, "r") as f:
        text = [line for line in f]
        host_filled = True
        user_filled = True
        for line in text:
            # Skip the commented lines
            if line.count("#"):
                continue
            if line.lower().count("host "):
                if not host_filled:
                   _host.append("-")
                if not user_filled:
                   _user.append("-")
                _name.append(line.split()[-1])
                host_filled = False
                user_filled = False
            if line.lower().count("hostname"):
                _host.append(line.split()[-1])
                host_filled = True
            if line.lower().count("user "):
                _user.append(line.split()[-1])
                user_filled = True
        if not host_filled:
           _host.append("-")
        if not user_filled:
           _user.append("-")
        assert len(_name) == len(_host) and len(_name) == len(_user), "Problem with ssh config file"  

    # Flag to check if at least one of the defined hosts has access.
    any_access = False
    # Initialize dictionary.
    accesses = {}
    # For all the different connection names, user-names and hosts, test if the password-less connection works.
    for i in range(len(_name)):
        if _host[i].count("mn1.bsc.es")+_host[i].count("mn2.bsc.es")+_host[i].count("mn3.bsc.es"):
            # Command to test if it is possible to connect without password to marenostrum.
            command = ['ssh -oNumberOfPasswordPrompts=0 %s "echo Working " &> /dev/null' % _name[i]]
           #command = ['ssh -oNumberOfPasswordPrompts=0 %s "echo Working " ' % _name[i]]
            # If return status is 0 save it to the dictionary
            stderr = open("passwordless.out","w")
            status = call(command,stderr=stderr,stdout=stderr, shell=True)
            if not status:
                accesses[_name[i]] = _user[i], _host[i]
                # Mark that at least there's one good acces
                any_access = True
    # If any one of the accesses work return None, otherwise return the valid access configurations.
            stderr.close()
            os.remove("passwordless.out")
    if not any_access:
            return None
    else:
            return accesses


# Function to return a single valid password-less configuration
def single_passworless_access():
    # Get all valid configurations
    _a = all_passwordless_access()
    # If there's at least one valid config
    if _a:
        # Check unique user-names
        _users = list(set([_x[0] for _x in _a.itervalues()]))
        # If there's only one username, return this configuration
        if len(_users) == 1:
            for _key in _a.keys():
                _user, _host = _a[_key]
                return _key, _user, _host
        # If there's more than one valid username, return the first username belonging to bsc32.
        else:
            for _key in _a.keys():
                _user, _host = _a[_key]
                if _user.count("bsc32"):
                    return _key, _user, _host
            return None, None, None
    else:
        print "No configured password-less access to Marenostrum 3"
        exit(1)


def check_return(val):
    if val:
        print "Error!Exiting..."
        exit(val)


def send(filename):
    debug = False
    isfile = os.path.isfile
    isdir = os.path.isdir
    assert isfile(filename) or isdir(filename), "Is not a filename nor directory"

    _name, _user, _host = single_passworless_access()

    if not _name:
        print "More than one username found to access Marenostrum 3."

    if isfile(filename):
        check_return(call(["scp %s %s:~ &> /dev/null" % (filename, _name)], shell=True))
    elif isdir(filename):
        if debug: print "Compressing...%s" % filename
        t0 = time.time()
        check_return(call(["tar -cf to_send.tar %s &> /dev/null" % filename], shell=True))
        t1 = time.time()
        if debug: print "Compressed in %.2f seconds" % (t1 - t0)
        if debug: print "Sending...%s" % filename
        check_return(call(["scp -r %s %s:~ &> /dev/null" % ("to_send.tar", _name)], shell=True))
        check_return(call(["rm to_send.tar"], shell=True))
        t2 = time.time()
        if debug: print "Sent in %.2f seconds" % (t2-t1)
        if debug: print "Inflating...%s" % filename
        check_return(call(['ssh %s "tar -xf ~/to_send.tar && rm  ~/to_send.tar"' % _name], shell=True))
        t3 = time.time()
        if debug: print "Inflated in %.2f seconds" % (t3-t2)
        if debug: print "Directory %s sent to Marenostrum" % filename
    else:
        print "Whaaaaat?"


def test_working_version():
    _name, _user, _host = single_passworless_access()
    if not _name:
        print "More than one username found to access Marenostrum 3."
    check_return(call(['ssh %s "ls -lrt ~/ELPiN/bin/find_sea_procs.exe"' % _name], shell=True))

if __name__ == "__main__":
    name, user, host = single_passworless_access()
    if name:
        print name
    else:
        a = all_passwordless_access()
        if a:
            users = list(set([x[0] for x in a.itervalues()]))
            for key in a.keys():
                user, host = a[key]
                print "%11s\t->\t%s@%s" % (key, user, host)
        else:
            print "No configured passwordless access to Marenostrum 3"

